﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data.EntityClient;
using UniversalDataAccess.BusinessEntities;

namespace UniversalDataAccess
{
    public class SectionDataAccess : OrganizationalDivisionsDataAccess
    {
        public SectionDataAccess(string dbName, string conn, bool usePredefinedMetadata)
            :base(dbName, conn, usePredefinedMetadata)
        {
        }

        public List<Section> SelectSectionsList()
        {
            try
            {
                //Select OrganizationalDivision List
                List<OrganizationalDivision> orgDivList = SelectOrganizationalDivisionsList();

                //Select Sections List
                List<SECTION> sectionList = dataContext.SECTION.ToList();

                //Merge result
                var res = (from orgDiv in orgDivList
                           join section in sectionList on orgDiv.ID equals section.ID
                           select new Section
                          {
                              DataBaseName = dataBaseName,
                              CLASS_ID = orgDiv.CLASS_ID,
                              EXT_ID = orgDiv.EXT_ID,
                              ID = orgDiv.ID,
                              NAME = orgDiv.NAME,
                              PARENT_ID = orgDiv.PARENT_ID,
                              BOSS_ID = orgDiv.BOSS_ID,
                              DATE_CREATED = orgDiv.DATE_CREATED,
                              DESCRIPTION = orgDiv.DESCRIPTION,
                              EMAIL = orgDiv.EMAIL,
                              SLOGAN = orgDiv.SLOGAN,
                              ROOM_ID = section.ROOM_ID,
                              LOCATION_ID = section.LOCATION_ID,
                              SCHEDULE = section.SCHEDULE
                          }).ToList();

                return res;
            }
            catch
            {
                throw new Exception("Error while SelectSectionBy_Id");
            }
        }

        public Section SelectSectionBy_Id(int Id)
        {
            try
            {
                //Select OrgDiv part by Id
                OrganizationalDivision orgDivPart = SelectOrganizationalDivisionBy_Id(Id);

                //Select Section part
                List<SECTION>  sectionPart = dataContext.SECTION.ToList();

                //Merge result
                var res = (from section in sectionPart
                           where section.ID == orgDivPart.ID
                           select new Section
                           {
                               DataBaseName = dataBaseName,
                               CLASS_ID = orgDivPart.CLASS_ID,
                               EXT_ID = orgDivPart.EXT_ID,
                               ID = orgDivPart.ID,
                               NAME = orgDivPart.NAME,
                               PARENT_ID = orgDivPart.PARENT_ID,
                               BOSS_ID = orgDivPart.BOSS_ID,
                               DATE_CREATED = orgDivPart.DATE_CREATED,
                               DESCRIPTION = orgDivPart.DESCRIPTION,
                               EMAIL = orgDivPart.EMAIL,
                               SLOGAN = orgDivPart.SLOGAN,
                               ROOM_ID = section.ROOM_ID,
                               LOCATION_ID = section.LOCATION_ID,
                               SCHEDULE = section.SCHEDULE
                           }).FirstOrDefault();

                return res;
            }
            catch
            {
                throw new Exception("Error while SelectSectionBy_Id");
            }
        }

        public void InsertSection(Section section, ref int generated_Id)
        {
            try
            {
                //Insert the OrgDivision part
                InsertOrganizationalDivision(section, ref generated_Id);

                //Insert the Section part
                int new_Id = generated_Id;
                SECTION sectionPart = new SECTION();
                sectionPart.ORGANIZATIONAL_DIVISION = dataContext.ORGANIZATIONAL_DIVISION.Where(t => t.ID == new_Id).FirstOrDefault();
                sectionPart.ID = new_Id;
                sectionPart.ROOM_ID = section.ROOM_ID;
                sectionPart.LOCATION_ID = section.LOCATION_ID;
                sectionPart.SCHEDULE = section.SCHEDULE;
               
                //Submit changes
                dataContext.AddToSECTION(sectionPart);
                dataContext.SaveChanges();
            }
            catch(Exception ex)
            {
                throw new Exception("Error while InsertSection");
            }
        }

        public void UpdateSection(Section section)
        {
            try
            {
                //Update the OrganizationalDivision part
                UpdateOrganizationalDivision(section);

                //Update the Section part
                SECTION sectionPart = dataContext.SECTION.Where(t => t.ID == section.ID).FirstOrDefault();
                sectionPart.ROOM_ID = section.ROOM_ID;
                sectionPart.LOCATION_ID = section.LOCATION_ID;
                sectionPart.SCHEDULE = section.SCHEDULE;
               
                //Submit changes
                dataContext.SaveChanges();
            }
            catch(Exception ex)
            {
                throw new Exception("Error while UpdateSection");
            }
        }

        public void DeleteSection(Section section)
        {
            try
            {
                //Delete the Section part
                SECTION sectionPart = dataContext.SECTION.Where(t => t.ID == section.ID).FirstOrDefault();

                dataContext.DeleteObject(sectionPart);
                dataContext.SaveChanges();

                //Delete the OrganizationalDivision part
                DeleteOrganizationalDivision(section);
            }
            catch (Exception ex)
            {
                throw new Exception("Error while DeleteSection");
            }
        }
    }
}
